// -----------------------------------------------------------------------
// Copyright (C) 2019-2023, EyeLogic GmbH
//
// Permission is hereby granted, free of charge, to any person or
// organization obtaining a copy of the software and accompanying
// documentation covered by this license (the "Software") to use,
// reproduce, display, distribute, execute, and transmit the Software,
// and to prepare derivative works of the Software, and to permit
// third-parties to whom the Software is furnished to do so.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
// NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE
// DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER
// LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
// OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// -----------------------------------------------------------------------

#include "elcapi/ELCApi.h"

#include <stdio.h>
#ifdef _WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

void
closeDemo( )
{
    elDestroyApi( );
    printf( "\npress enter to exit\n" );
    getchar( );
}

void STDCALL
onGazeSample( const struct ELCGazeSample* gazeSample )
{
    printf( "new gaze sample - L: " );
    if ( gazeSample->porLeftX != ELCInvalidValue ) {
        printf( "%f / %f", gazeSample->porLeftX, gazeSample->porLeftY );
    } else {
        printf( "invalid" );
    }

    printf( " - R: " );
    if ( gazeSample->porRightX != ELCInvalidValue ) {
        printf( "%f / %f", gazeSample->porRightX, gazeSample->porRightY );
    } else {
        printf( "invalid" );
    }

    printf( "\n                  binocular: " );
    if ( gazeSample->porRawX != ELCInvalidValue ) {
        printf( "%f / %f", gazeSample->porRawX, gazeSample->porRawY );
    } else {
        printf( "invalid" );
    }

    printf( " - filtered: " );
    if ( gazeSample->porFilteredX != ELCInvalidValue ) {
        printf( "%f / %f", gazeSample->porFilteredX, gazeSample->porFilteredY );
    } else {
        printf( "invalid" );
    }
    printf( "\n" );
}

void STDCALL
onEvent( enum ELCEvent event )
{
    switch ( event ) {
        case EVENT_SCREEN_CHANGED: {
            printf( "active screen has changed\n" );
            struct ELCScreenConfig screenConfig;
            elGetActiveScreen( &screenConfig );
            printf( "    new screen: resolution = %4dx%4d\n",
                    screenConfig.resolutionX,
                    screenConfig.resolutionY );
            return;
        }
        case EVENT_CONNECTION_CLOSED:
            printf( "server has closed the connection\n" );
            return;
        case EVENT_DEVICE_CONNECTED:
            printf( "device has connected\n" );
            return;
        case EVENT_DEVICE_DISCONNECTED:
            printf( "device has disconnected\n" );
            return;
        case EVENT_TRACKING_STOPPED:
            printf( "tracking has stopped\n" );
            return;
    }
}

int
main( )
{
    // connect to server
    elInitApi( "Gaze Sample Demo Client" );

    // register event callback
    elRegisterEventCallback( onEvent );

    const enum ELCReturnConnect retConnect = elConnect( );
    if ( retConnect == CONNECT_SUCCESS ) {
        printf( "connected to server\n" );
    } else {
        printf( "Cannot connect to server: elConnect failed. Is the server running?\n" );
        closeDemo( );
        return 1;
    }

    // check screen resolution
    struct ELCScreenConfig screenConfig;
    elGetActiveScreen( &screenConfig );
    printf( "screen resolution: %4dx%4d\n", screenConfig.resolutionX, screenConfig.resolutionY );

    // check connected eye tracker
    struct ELCDeviceConfig deviceConfig;
    elGetDeviceConfig( &deviceConfig );
    if ( deviceConfig.deviceSerial == 0 ) {
        printf( "no device connected" );
        closeDemo( );
        return 1;
    }
    printf( "device found, supported framerates: " );
    for ( int i = 0; i < deviceConfig.numFrameRates; i++ ) {
        if ( i > 0 ) {
            printf( ", " );
        }
        printf( "%d4", deviceConfig.frameRates[ i ] );
    }
    printf( "\n" );

    // start tracking
    const enum ELCReturnStart retStart = elRequestTracking( 0 );
    if ( retStart == START_SUCCESS ) {
        printf( "tracking started\n" );
    } else {
        printf( "cannot start tracking: elRequestTracking failed\n" );
        closeDemo( );
        return 1;
    }

    // calibrate
    const enum ELCReturnCalibrate retCalibrate = elCalibrate( 0 );
    if ( retCalibrate == CALIB_SUCCESS ) {
        printf( "calibrated\n" );
    } else {
        printf( "could not calibrate: elCalibrate failed\n" );
        closeDemo( );
        return 1;
    }

    // register gaze sample callback
    elRegisterGazeSampleCallback( onGazeSample );

#ifdef _WIN32
    Sleep( 10000 /* ms */ );
#else
    sleep( 10 /* s */ );
#endif

    // unregister callbacks
    elRegisterGazeSampleCallback( 0 );
    elRegisterEventCallback( 0 );

    // disconnect
    printf( "disconnecting...\n" );
    closeDemo( );
    return 0;
}
