// -----------------------------------------------------------------------
// Copyright (C) 2019-2023, EyeLogic GmbH
//
// Permission is hereby granted, free of charge, to any person or
// organization obtaining a copy of the software and accompanying
// documentation covered by this license (the "Software") to use,
// reproduce, display, distribute, execute, and transmit the Software,
// and to prepare derivative works of the Software, and to permit
// third-parties to whom the Software is furnished to do so.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
// NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE
// DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER
// LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
// OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// -----------------------------------------------------------------------

/**
 * Client demonstrating the use of remote connection.
 * Setup:
 *   - Computer A is running the EyeLogic Server. This is the workplace of the
 *     operator of the experiment.
 *   - Computer B is running the experiment. It is shown on a screen connected
 *     to computer B. This is the workplace of the participant.
 *   - The eye tracker is mounted on that screen (connected to computer B).
 *   - The eye tracker is wired to the USB port of computer A.
 *
 * In this setup, the client (running on computer B) has to connect to the
 * EyeLogic Server on computer A via the call connectRemote(). Both computers
 * need to be in the same local network.
 */

#include "elcapi/ELCApi.h"

#include <stdio.h>

void
closeDemo( )
{
    elDestroyApi( );
    printf( "\npress enter to exit\n" );
    getchar( );
}

void STDCALL
onEvent( enum ELCEvent event )
{
    switch ( event ) {
        case EVENT_SCREEN_CHANGED: {
            printf( "active screen has changed\n" );
            struct ELCScreenConfig screenConfig;
            elGetActiveScreen( &screenConfig );
            printf( "    new screen: %s (%4dx%4d)\n",
                    screenConfig.id,
                    screenConfig.resolutionX,
                    screenConfig.resolutionY );
            return;
        }
        case EVENT_CONNECTION_CLOSED:
            printf( "server has closed the connection\n" );
            return;
        case EVENT_DEVICE_CONNECTED:
            printf( "device has connected\n" );
            return;
        case EVENT_DEVICE_DISCONNECTED:
            printf( "device has disconnected\n" );
            return;
        case EVENT_TRACKING_STOPPED:
            printf( "tracking has stopped\n" );
            return;
    }
}

int
main( )
{
    // initialize API
    elInitApi( "Demo Client DualPC" );

    // register event callback
    elRegisterEventCallback( onEvent );

    // obtain list of available EyeLogic servers in the local network
    // Note: a server has to "enable remote" connections in order to be found
    struct ELCServerInfo serverList[ 4 ];  // the choice of buffer length is arbitrary
    const int            numServers = elRequestServerList( 120, serverList, 4 );
    if ( numServers == 0 ) {
        printf(
            "No server on local PC or in LAN found. Server must be running and "
            "has 'allow remote' connections enabled.\n" );
        closeDemo( );
        return 1;
    }

    printf( "found %d server(s):\n", numServers );
    for ( int i = 0; i < numServers; i++ ) {
        printf( "[%d] IP: %s\n", i, serverList[ i ].ip );
    }

    // connect to server
    printf( "connecting to server [0]\n" );
    const enum ELCReturnConnect retConnect = elConnectRemote( serverList[ 0 ] );
    if ( retConnect == CONNECT_SUCCESS ) {
        printf( "connected to server\n" );
    } else {
        printf( "Cannot connect to server: elConnectRemote failed. Is the server running?\n" );
        closeDemo( );
        return 1;
    }

    // get available screens on this PC
    struct ELCScreenConfig screenConfig[ 4 ];
    const int              numAvailableScreens = elGetAvailableScreens( screenConfig, 4 );
    if ( numAvailableScreens == 0 ) {
        printf( "no screens connected to local PC" );
        closeDemo( );
        return 1;
    }
    printf( "found %d screen(s):\n", numAvailableScreens );
    for ( int i = 0; i < numAvailableScreens; i++ ) {
        printf( "[%d] ID: %s (%dx%d)\n",
                i,
                screenConfig[ i ].id,
                screenConfig[ i ].resolutionX,
                screenConfig[ i ].resolutionY );
    }

    // set active screen
    struct ELCDeviceGeometry deviceGeometry;
    deviceGeometry.mmBelowScreen            = 10;
    deviceGeometry.mmTrackerInFrontOfScreen = 10;

    printf(
        "Using screen [0]. Assuming eye tracker position (reference point): "
        "%fmm below, %fmm in front\n",
        deviceGeometry.mmBelowScreen,
        deviceGeometry.mmTrackerInFrontOfScreen );
    elSetActiveScreen( screenConfig[ 0 ].id, deviceGeometry );

    // check connected eye tracker
    struct ELCDeviceConfig deviceConfig;
    elGetDeviceConfig( &deviceConfig );
    if ( deviceConfig.deviceSerial == 0 ) {
        printf( "no device connected\n" );
        closeDemo( );
        return 1;
    }
    printf( "device found, supported framerates: " );
    for ( int i = 0; i < deviceConfig.numFrameRates; i++ ) {
        if ( i > 0 ) {
            printf( ", " );
        }
        printf( "%4d", deviceConfig.frameRates[ i ] );
    }
    printf( "\n" );

    // start tracking
    const enum ELCReturnStart retStart = elRequestTracking( 0 );
    if ( retStart == START_SUCCESS ) {
        printf( "tracking started\n" );
    } else {
        elDestroyApi( );
        printf( "cannot start tracking: elRequestTracking failed\n" );
        closeDemo( );
        return 1;
    }

    // calibrate
    const enum ELCReturnCalibrate retCalibrate = elCalibrate( 0 );
    if ( retCalibrate == CALIB_SUCCESS ) {
        printf( "calibrated\n" );
    } else {
        elDestroyApi( );
        printf( "could not calibrate: elCalibrate failed\n" );
        closeDemo( );
        return 1;
    }

    // calibration successful
    // run your experiment here
    printf( "EyeLogic device is up and running\n" );

    // unregister callbacks
    elRegisterEventCallback( 0 );

    // disconnect
    printf( "disconnecting...\n" );
    closeDemo( );
    return 0;
}
