// -----------------------------------------------------------------------
// Copyright (C) 2019-2023, EyeLogic GmbH
//
// Permission is hereby granted, free of charge, to any person or
// organization obtaining a copy of the software and accompanying
// documentation covered by this license (the "Software") to use,
// reproduce, display, distribute, execute, and transmit the Software,
// and to prepare derivative works of the Software, and to permit
// third-parties to whom the Software is furnished to do so.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
// NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE
// DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER
// LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
// OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// -----------------------------------------------------------------------

#include "elcapi/ELCApi.h"

#include <stdio.h>
#include <math.h>
#ifdef _WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

void
closeDemo( )
{
    elRegisterEventCallback( 0 );
    elRegisterGazeSampleCallback( 0 );
    elDestroyApi( );
    printf( "\npress enter to exit\n" );
    getchar( );
}

void
printValidationData( const struct ELCValidationResult* validationResult )
{
    printf( "validated\n" );
    for ( int i = 0; i < 4; i++ ) {
        char leftDeg[ 10 ], leftPx[ 10 ], rightDeg[ 10 ], rightPx[ 10 ];

        if ( validationResult->pointsData[ i ].meanDeviationLeftDeg == ELCInvalidValue ) {
            // degree and pixel deviations will always be either both valid or both invalid
            snprintf( leftDeg, 10, "N/A" );
            snprintf( leftPx, 10, "N/A" );
        } else {
            snprintf(
                leftDeg, 10, "%4.1f deg", validationResult->pointsData[ i ].meanDeviationLeftDeg );
            snprintf(
                leftPx, 10, "%4.1f px", validationResult->pointsData[ i ].meanDeviationLeftPx );
        }

        // degree and pixel deviations will always be either both valid or both invalid
        if ( validationResult->pointsData[ i ].meanDeviationRightDeg == ELCInvalidValue ) {
            snprintf( rightDeg, 10, "N/A" );
            snprintf( rightPx, 10, "N/A" );
        } else {
            snprintf( rightDeg,
                      10,
                      "%4.1f deg",
                      validationResult->pointsData[ i ].meanDeviationRightDeg );
            snprintf(
                rightPx, 10, "%4.1f px", validationResult->pointsData[ i ].meanDeviationRightPx );
        }

        printf( "%d. position [px]: %4d, %4d  |  left: %9s / %9s  |  right: %9s / %9s\n",
                i + 1,
                (int)round( validationResult->pointsData[ i ].validationPointPxX ),
                (int)round( validationResult->pointsData[ i ].validationPointPxY ),
                leftDeg,
                leftPx,
                rightDeg,
                rightPx );
    }
}

int
main( )
{
    // connect to server
    elInitApi( "Validation Demo Client" );

    const enum ELCReturnConnect retConnect = elConnect( );
    if ( retConnect == CONNECT_SUCCESS ) {
        printf( "connected to server\n" );
    } else {
        printf( "cannot connect to server: elConnect failed. Is the server running?" );
        closeDemo( );
        return 1;
    }

    // check screen resolution
    struct ELCScreenConfig screenConfig;
    elGetActiveScreen( &screenConfig );
    printf( "screen resolution: %4dx%4d\n", screenConfig.resolutionX, screenConfig.resolutionY );

    // check connected eye tracker
    struct ELCDeviceConfig deviceConfig;
    elGetDeviceConfig( &deviceConfig );
    if ( deviceConfig.deviceSerial == 0 ) {
        printf( "no device connected" );
        closeDemo( );
        return 1;
    }
    printf( "device found, supported framerates: " );
    for ( int i = 0; i < deviceConfig.numFrameRates; i++ ) {
        if ( i > 0 ) {
            printf( ", " );
        }
        printf( "%4d", deviceConfig.frameRates[ i ] );
    }
    printf( "\n" );

    // start tracking
    const enum ELCReturnStart retStart = elRequestTracking( 0 );
    if ( retStart == START_SUCCESS ) {
        printf( "tracking started\n" );
    } else {
        printf( "cannot start tracking: elRequestTracking failed\n" );
        closeDemo( );
        return 1;
    }

    // calibrate
    const enum ELCReturnCalibrate retCalibrate = elCalibrate( 0 );
    if ( retCalibrate == CALIB_SUCCESS ) {
        printf( "calibrated\n" );
    } else {
        printf( "could not calibrate: elCalibrate failed\n" );
        closeDemo( );
        return 1;
    }

    // validate
    struct ELCValidationResult   validationResult;
    const enum ELCReturnValidate retValidate = elValidate( &validationResult );
    if ( !retValidate == VALID_SUCCESS ) {
        printf( "could not validate: elValidate failed\n" );
        closeDemo( );
        return 1;
    }

    // show validation results
    printValidationData( &validationResult );

    // disconnect
    printf( "disconnecting...\n" );
    closeDemo( );
    return 0;
}
