// -----------------------------------------------------------------------
// Copyright (C) 2019-2023, EyeLogic GmbH
//
// Permission is hereby granted, free of charge, to any person or
// organization obtaining a copy of the software and accompanying
// documentation covered by this license (the "Software") to use,
// reproduce, display, distribute, execute, and transmit the Software,
// and to prepare derivative works of the Software, and to permit
// third-parties to whom the Software is furnished to do so.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
// NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE
// DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER
// LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
// OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// -----------------------------------------------------------------------

/**
 * @file ELCApi.h
 *
 * @brief The file contains the C prototype declaration for all functions which are
 * neccessary to control the EyeLogic software from an API client.
 */

#pragma once

#include "ELCGazeSample.h"
#include "ELCEyeImage.h"

/** @brief EyeLogic events */
enum ELCEvent {
    /** @brief a new screen has been set as active */
    EVENT_SCREEN_CHANGED,
    /** @brief connection to EyeLogic Server has closed */
    EVENT_CONNECTION_CLOSED,
    /** @brief a new device has connected */
    EVENT_DEVICE_CONNECTED,
    /** @brief device disconnected */
    EVENT_DEVICE_DISCONNECTED,
    /** @brief tracking has stopped */
    EVENT_TRACKING_STOPPED
};

/** @brief callback function type, new gaze sample */
typedef void( STDCALL* GazeSampleCallback )( const struct ELCGazeSample* sample );
/** @brief callback function type, new eye image */
typedef void( STDCALL* EyeImageCallback )( const struct ELCEyeImage* eyeImage );
/** @brief callback function type, connection closed by server */
typedef void( STDCALL* EventCallback )( enum ELCEvent );

/** @brief return values of elInitApi( ) */
enum ELCReturnInit {
    /** @brief initializeation was successful */
    INIT_SUCCESS,
    /** @brief initialization was not successfull: was already initialized before */
    INIT_ALREADY_INITED
};

/**
 * @brief initialize the ELCApi.
 *
 * Call this once before calling any other function from this library.
 *
 * @param   clientName          string identifier of the client (shown in the server tool window)
 */
EXTERNC ELC_EXPORT enum ELCReturnInit STDCALL elInitApi( const char* clientName );

/**
 * @brief destroys the ELCApi. Call this once before shutting down. Be sure to unregister all
 * callbacks before calling this method.
 *
 * @see elRegisterEventCallback( )
 * @see elRegisterGazeSampleCallback( )
 * @see elRegisterEyeImageCallback( )
 */
EXTERNC ELC_EXPORT void STDCALL elDestroyApi( );

/**
 * @brief registers the callback receiver for events.
 * Ensure that all callbacksare unregistered before destruction.
 *
 * @param   eventCallback      this callback function is called on new events,
 *                             may be null to unregister
 */
EXTERNC ELC_EXPORT void STDCALL elRegisterEventCallback( EventCallback eventCallback );

/**
 * @brief registers the callback receiver for gaze samples.
 * Ensure that all callbacksare unregistered before destruction.
 *
 * @param   sampleCallback      this callback function is called on new gaze samples,
 *                              may be null to unregister
 */
EXTERNC ELC_EXPORT void STDCALL elRegisterGazeSampleCallback( GazeSampleCallback sampleCallback );

/**
 * @brief registers the callback receiver for eye images.
 * Ensure that all callbacksare unregistered before destruction.
 *
 * @param   eyeImageCallback    this callback function is called on new eye images,
 *                              may be null to unregister
 */
EXTERNC ELC_EXPORT void STDCALL elRegisterEyeImageCallback( EyeImageCallback eyeImageCallback );

/** @brief return values of elConnect( ) */
enum ELCReturnConnect {
    /** @brief connection successfully established */
    CONNECT_SUCCESS,
    /** @brief connection failed: library needs to be initialized first, @see elInitApi */
    CONNECT_NOT_INITED,
    /** @brief connection failed: API is build on a newer version than the server. Update the
       EyeLogicServer to the newest version. */
    CONNECT_VERSION_MISMATCH,
    /** @brief connection failed: the server can not be found or is not responding */
    CONNECT_FAILURE
};
/**
 * @brief return values of elConnect( )
 * @returns success state
 */
EXTERNC ELC_EXPORT enum ELCReturnConnect STDCALL elConnect( );

/**
 * @brief initialize connection to the server (method is blocking until connection
 * established). The connection is only established for a local server (running on this
 * machine). For connections to a remote server, @see elConnectRemote( ).
 *
 * @return  success state
 */
EXTERNC ELC_EXPORT enum ELCReturnConnect STDCALL elConnect( );

/** @brief connection information for an EyeLogic server */
EXTERNC struct ELCServerInfo {
    /** @brief IP address of server as 0-terminated string */
    char ip[ 16 ];
    /** @brief port of server */
    uint16_t port;
};

/**
 * @brief initialize connection to a remote server (method is blocking until connection
 * established)
 *
 * @param   server  Server to connect to
 *
 * @return  success state
 *
 * @see elAcquireServerList( ) to obtain IP address and port of a remote server
 */
EXTERNC ELC_EXPORT enum ELCReturnConnect STDCALL elConnectRemote( struct ELCServerInfo server );

/**
 * @brief Ping all running EyeLogic servers in the local network and wait some time for their
 * response.
 *
 * @param   blockingDurationMS  waiting duration in milliseconds. Method returns after this
 *                              time, or if 'serverListLength' many servers responded.
 * @param   serverList          pre-allocated array of length 'serverListLength'. Will be filled
 *                              with responding EyeLogic servers.
 * @param   serverListLength    Lenght of pre-allocated serverList array
 *
 * @return  number of entries, written to the server list
 */
EXTERNC ELC_EXPORT int32_t STDCALL elRequestServerList( int32_t               blockingDurationMS,
                                                        struct ELCServerInfo* serverList,
                                                        int32_t               serverListLength );

/** @brief closes connection to the server */
EXTERNC ELC_EXPORT void STDCALL elDisconnect( );

/** @brief whether a connection to the server is established */
EXTERNC ELC_EXPORT bool STDCALL elIsConnected( );

/** @brief Screen configuration */
EXTERNC struct ELCScreenConfig {
    /** @brief whether the screen is connected to the this machine */
    bool localMachine;
    /** @brief identifier name of the screen (0-terminated string) */
    char id[ 32 ];
    /** @brief descriptive name of the screen (0-terminated string) */
    char name[ 32 ];
    /** @brief screen X resolution [px] */
    int32_t resolutionX;
    /** @brief screen Y resolution [px] */
    int32_t resolutionY;
    /** @brief horizontal physical dimension of the screen [mm] */
    double physicalSizeX_mm;
    /** @brief vertical physical dimension of the screen [mm] */
    double physicalSizeY_mm;
};
/** @brief obtain configuration of active screen
 *
 * @param screenConfig is set to the config state upon success, set to 0-initialized struct if
 * library is not initialized
 *
 * @return true upon successfull config recovery, false upon unitialized library or screenConfig ==
 * nullptr
 */
EXTERNC ELC_EXPORT bool STDCALL elGetActiveScreen( struct ELCScreenConfig* screenConfig );

/**
 * @brief Get a list of screens connected to the local machine. If there are more screens than
 * 'numScreenConfigs' found, then only the first 'numScreenConfigs' ones are filled.
 *
 * @param   screenConfig        pre-allocated array, will be filled with screen configurations
 * @param   numScreenConfigs    number of entries of screenConfig
 * @return  number of filled screen configurations. will be <= numScreenConfigs
 */
EXTERNC ELC_EXPORT int32_t STDCALL elGetAvailableScreens( struct ELCScreenConfig* screenConfig,
                                                          int32_t numScreenConfigs );

/** @brief Geometric position of the device related to the active monitor */
EXTERNC struct ELCDeviceGeometry {
    /**
     * @brief vertical distance between the lowest pixel on the display and the upper edge of
     * the eye tracker
     */
    double mmBelowScreen;
    /**
     * @brief horizontal distance between the front of the screen and the front edge of the eye
     * tracker
     */
    double mmTrackerInFrontOfScreen;
};

/** @brief return values of setActiveScreen( ) */
enum ELCReturnActiveScreen {
    /** active screen was set */
    ACTIVESCREEN_SUCCESS,
    /** @brief library needs to be initialized first, @see elInitApi */
    ACTIVESCREEN_NOT_INITED,
    /** specified screen name was not found as a name of an available monitor */
    ACTIVESCREEN_NOT_FOUND,
    /** active screen could not be changed */
    ACTIVESCREEN_FAILURE
};

/**
 * @brief Make a screen connected to this machine to the active screen.
 *
 * Recording is from now on performed on the new active screen. Remember to perform a
 * calibration on the new screen, otherwise it remains in an uncalibrated state.
 *
 * @param   screenID  ID of the new active screen on _this_ machine (even works if the
 *                    connection to the server is remote).
 *                    If null, the primary screen of this machine is set as active.
 * @param   deviceGeometry  Geometry of the device which is mounted to the screen.
 * @return  success/error code
 */
EXTERNC ELC_EXPORT enum ELCReturnActiveScreen STDCALL
elSetActiveScreen( char* screenID, struct ELCDeviceGeometry deviceGeometry );

/** @brief Device configuration */
EXTERNC struct ELCDeviceConfig {
    /** @brief serial number of the device as unsigned 64-bit int
     * for a verbose format, print it as 8-digit hex number */
    uint64_t deviceSerial;
    /** @brief number of available framerates */
    int32_t numFrameRates;
    /** @brief array of available framerates [Hz], use only the entries
     * frameRates[0] ... frameRates[numFrameRates-1] */
    uint8_t frameRates[ 16 ];
    /** @brief number of available calibration methods */
    int32_t numCalibrationMethods;
    /** @brief array of available calibration methods [number of calibration points],
     * use only the entries
     * calibrationMethods[0] ... calibrationMethods[numCalibrationMethods-1] */
    uint8_t calibrationMethods[ 16 ];
};

/** @brief obtain configuration of active device
 *
 * @param deviceConfig is set to the config state upon success, set to 0-initialized struct if
 * library is not initialized
 *
 * @return true upon successfull config recovery, false upon unitialized library or screenConfig ==
 * nullptr
 */
EXTERNC ELC_EXPORT bool STDCALL elGetDeviceConfig( struct ELCDeviceConfig* deviceConfig );

/** @brief Return values of the elStreamEyeImages() function. */
enum ELCReturnStreamEyeImages {
    /** @brief setting streaming of eye images was successful */
    EYEIMAGES_SUCCESS,
    /** @brief failed, not connected to the server */
    EYEIMAGES_NOT_CONNECTED,
    /** @brief cannot stream eye images when connection to the server is a remote connection */
    EYEIMAGES_REMOTE_CONNECTION,
    /** @brief failure when trying to set eye image stream */
    EYEIMAGES_FAILURE
};

/**
 * @brief Enabled/disables eye image stream. If enabled, eye images are received from eye image
 * listeners, @see registerEyeImageListener() and @see getNextEyeImage(). Note, that enabling
 * eye images can lead to noticable CPU load and a loss of gaze samples. Always disable it
 * before running your experiment. Eye images can not be received via remote connections.
 */
EXTERNC ELC_EXPORT enum ELCReturnStreamEyeImages STDCALL elStreamEyeImages( bool enable );

/** @brief Return values of the getNextEvent/getNextGazeSample functions. */
enum ELCReturnNextData {
    /** @brief new event or new GazeSample received */
    NEXTDATA_SUCCESS,
    /** @brief input ptr is nullptr */
    NEXTDATA_PTR_IS_NULL,
    /** @brief library needs to be initialized first, @see elInitApi */
    NEXTDATA_NOT_INITED,
    /** @brief timeout reached, no new event/GazeSample received */
    NEXTDATA_TIMEOUT,
    /** @brief connection to server closed, no new event/GazeSample received */
    NEXTDATA_CONNECTION_CLOSED
};

/**
 * @brief Obtains the next unread event or blocks until a new event occurs or the given timeout
 * is reached.
 *
 * The last incoming event is buffered internally and can be obtained by calling this method in
 * a consecutive order. If there is no new event, the method blocks until an event occurs or the
 * given timeout is reached. The method returns SUCCESS if and only if a new event is provided
 * which was not returned before. Therefore, by checking the return value, you can assure to not
 * handle any event twice.
 *
 * If you want to catch events in a loop, be careful to not wait too long between the calls to
 * this method. Otherwise, you may miss events. If you want to be 100% sure to not miss any
 * event, consider to use the ELEventCallback mechanism. @see registerEventListener
 *
 * @param   event           If this method returns SUCCESS, this data structure is filled with
 *                          the new (yet unhandled) event. In all other cases, this data
 *                          structure is filled with the event which was returned last.
 * @param   timeoutMillis   duration in milliseconds, method returns at the latest after this
 *                          time. May be 0 if the method should return immediatly.
 * @return  whether an event was received (SUCCESS) or the method terminated without a new event
 */
EXTERNC ELC_EXPORT enum ELCReturnNextData STDCALL elGetNextEvent( enum ELCEvent* event,
                                                                  int32_t        timeoutMillis );

/**
 * @brief Obtains the next unread gazeSample or blocks until a new GazeSample is received or the
 * given timeout is reached.
 *
 * The last incoming GazeSample is buffered internally and can be obtained by calling this
 * method in a consecutive order. If there is no new GazeSample, the method blocks until a
 * GazeSample arrives or the given timeout is reached. The method returns SUCCESS if and only if
 * a new GazeSample is provided which was not returned before. Therefore, by checking the return
 * value, you can assure to not handle any GazeSample twice.
 *
 * If you want to catch GazeSamples in a loop, be careful to not wait too long between the calls
 * to this method (at least once per frame). Otherwise, you may miss GazeSamples. If you want to
 * be 100% sure to not miss any GazeSample, consider to use the ELGazeSampleCallback mechanism.
 * @see registerGazeSampleListener
 *
 * @param   gazeSample      If this method returns SUCCESS, this data structure is filled with
 *                          the new (yet unhandled) GazeSample. In all other cases, this data
 *                          structure is filled with the GazeSample which was returned last.
 * @param   timeoutMillis   duration in milliseconds, method returns at the latest after this
 *                          time. May be 0 if the method should return immediatly.
 * @return  whether a GazeSample was received (SUCCESS) or the method terminated without a new
 *          GazeSample
 */
EXTERNC ELC_EXPORT enum ELCReturnNextData STDCALL
elGetNextGazeSample( struct ELCGazeSample* gazeSample, int32_t timeoutMillis );

/**
 * @brief Obtains the next unread eye image or blocks until a new eye image is received or the
 * given timeout is reached.
 *
 * The last incoming eye image is buffered internally and can be obtained by calling this method
 * in a consecutive order. If there is no new eye image, the method blocks until an eye image is
 * received or the given timeout is reached. The method returns SUCCESS if and only if a new
 * eye image is provided which was not returned before. Therefore, by checking the return value,
 * you can assure to not handle any eye image twice.
 *
 * @param   eyeImage        If this method returns SUCCESS, this data structure is filled with
 *                          the new (yet unhandled) eye image. In all other cases, this data
 *                          structure is filled with the eye image which was returned last.
 * @param   timeoutMillis   duration in milliseconds, method returns at the latest after this
 *                          time. May be 0 if the method should return immediatly.
 * @return  whether an eye iage was received (SUCCESS)
 */
EXTERNC ELC_EXPORT enum ELCReturnNextData STDCALL elGetNextEyeImage( struct ELCEyeImage* eyeImage,
                                                                     int32_t timeoutMillis );

/** @brief return values of elRequestTracking( ) */
enum ELCReturnStart {
    /** @brief start tracking successful */
    START_SUCCESS,
    /** @brief not connected to the server */
    START_NOT_CONNECTED,
    /** @brief cannot start tracking: no device found */
    START_DEVICE_MISSING,
    /** @brief cannot start tracking: framerate mode is invalid or not supported */
    START_INVALID_FRAMERATE_MODE,
    /** @brief tracking already ongoing, but frame rate mode is different */
    START_ALREADY_RUNNING_DIFFERENT_FRAMERATE,
    /** @brief some general failure occurred */
    START_FAILURE
};

/**
 * @brief request tracking
 *
 * If tracking is not yet ongoing, tracking is started in the device. If tracking is already
 * running (e.g. started from another client) with the same frame-rate as requested, all gaze
 * samples are reported to this client as well.
 *
 * @param   frameRateModeInd    index of the requested frame rate mode (0 .. \#frameRateModes-1)
 * @returns success state
 */
EXTERNC ELC_EXPORT enum ELCReturnStart STDCALL elRequestTracking( int32_t frameRateModeInd );

/**
 * @brief unrequest tracking
 *
 * Note that the tracking device may continue if other processes still request tracking. Check
 * the EyeLogic server window to observe the actual state.
 */
EXTERNC ELC_EXPORT void STDCALL elUnrequestTracking( );

/** @brief return values of elCalibrate( ) */
enum ELCReturnCalibrate {
    /** @brief calibration successful */
    CALIB_SUCCESS,
    /** @brief cannot calibrate: not connected to the server */
    CALIB_NOT_CONNECTED,
    /** @brief cannot calibrate: no device found or tracking not started */
    CALIB_NOT_TRACKING,
    /** @brief cannot start calibration: calibration mode is invalid or not supported */
    CALIB_INVALID_CALIBRATION_MODE,
    /** @brief cannot start calibration: calibration or validation is already in progress */
    CALIB_ALREADY_BUSY,
    /** @brief calibration was not successful or aborted */
    CALIB_FAILURE
};

/**
 * @brief perform calibration (method is blocking until calibration finished)
 *
 * @returns success state
 */
EXTERNC ELC_EXPORT enum ELCReturnCalibrate STDCALL elCalibrate( int32_t calibrationModeInd );

/** @brief  return values of elValidate( ) */
enum ELCReturnValidate {
    /** @brief start validation successful */
    VALID_SUCCESS,
    /** @brief cannot validate: not connected to the server */
    VALID_NOT_CONNECTED,
    /** @brief cannot validate: no device found or tracking not started */
    VALID_NOT_TRACKING,
    /** @brief cannot start validation: validation mode is invalid or not supported */
    VALID_NOT_CALIBRATED,
    /** @brief cannot start validation: calibration or validation is already in progress */
    VALID_ALREADY_BUSY,
    /** @brief input is nullptr */
    VALID_PTR_IS_NULL,
    /** @brief validation failure */
    VALID_FAILURE
};

/** @brief ValidationPointResult holds the results of the validation ( total deviation between
 * true point position and calculated POR of the left and right eye POR in [px] and [deg] ) of
 * the validation point at position ( validationPointPxX, validationPointPxY ) [px].
 *
 * The stimulus point position and deviation [px] are given in the 2D stimulus coordinate
 * system originating in the top left corner of the stimulus.
 *
 * The deviation [deg] corresponds to the total angular deviation between the measured gaze
 * direction from the ground truth gaze direction as determined according to the measured eye
 * position.
 *
 * Note: All data fields may be ELCInvalidValue. All pairs validationPointPxX/-Y,
 * meanDeviationLeftDeg/-Px and meanDeviationRightDeg-/Px are always either both valid or both
 * ELCInvalidValue.
 */
struct ELCValidationPointResult {
    /** @brief ELCInvalidValue or x-coordinate of stimulus point position */
    double validationPointPxX;
    /** @brief ELCInvalidValue or y-coordinate of stimulus point position */
    double validationPointPxY;
    /** @brief ELCInvalidValue or mean deviation between left eye POR and stimulus position in [px]
     * in the stimulus plane */
    double meanDeviationLeftPx;
    /** @brief ELCInvalidValue or mean deviation of left eye gaze direction in [deg] in the 3-D
     * world system */
    double meanDeviationLeftDeg;
    /** @brief ELCInvalidValue or mean deviation between right eye POR and stimulus position in [px]
     * in the stimulus plane */
    double meanDeviationRightPx;
    /** @brief ELCInvalidValue or mean deviation of right eye gaze direction in [deg] in the 3-D
     * world system */
    double meanDeviationRightDeg;
};

/** @brief ValidationResult contains one ValidationPointResult struct per validation stimulus
 * point of the performed valdation. */
struct ELCValidationResult {
    struct ELCValidationPointResult pointsData[ 4 ];
};

/**
 * @brief perform validation (method is blocking until validation finished) - calibration must be
 * performed prior
 *
 * @param validationResult upon ReturnValidate::SUCCESS this struct will be filled with the
 * validation results - may contain ELCInvalidValues. Contains all ELCInvalidValue for all other
 * return values - unless return value is VALID_NOT_CONNECTED, in that case validationResult will be
 * set to all-zeros.
 *
 * @returns success state
 */
EXTERNC ELC_EXPORT enum ELCReturnValidate STDCALL
elValidate( struct ELCValidationResult* validationResult );
