// -----------------------------------------------------------------------
// Copyright (C) 2019-2023, EyeLogic GmbH
//
// Permission is hereby granted, free of charge, to any person or
// organization obtaining a copy of the software and accompanying
// documentation covered by this license (the "Software") to use,
// reproduce, display, distribute, execute, and transmit the Software,
// and to prepare derivative works of the Software, and to permit
// third-parties to whom the Software is furnished to do so.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
// NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE
// DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER
// LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
// OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// -----------------------------------------------------------------------

#include "elapi/ELApi.h"

#include <iostream>
#include <chrono>
#include <thread>
#include <atomic>

void
close( )
{
    std::cout << std::endl << "press enter to continue" << std::endl;
    std::cin.ignore( );
}

class GazeSampleReceiver : public elapi::ELApi::ELGazeSampleCallback
{
    void STDCALL
    onGazeSample( const elapi::ELGazeSample& gazeSample ) override
    {
        std::cout << "new gaze sample - L: ";
        if ( gazeSample.porLeftX != elapi::ELInvalidValue ) {
            std::cout << gazeSample.porLeftX << " / " << gazeSample.porLeftY;
        } else {
            std::cout << "invalid";
        }

        std::cout << " - R: ";
        if ( gazeSample.porRightX != elapi::ELInvalidValue ) {
            std::cout << gazeSample.porRightX << " / " << gazeSample.porRightY;
        } else {
            std::cout << "invalid";
        }

        std::cout << std::endl << "                  binocular: ";
        if ( gazeSample.porRawX != elapi::ELInvalidValue ) {
            std::cout << gazeSample.porRawX << " / " << gazeSample.porRawY;
        } else {
            std::cout << "invalid";
        }

        std::cout << " - filtered: ";
        if ( gazeSample.porFilteredX != elapi::ELInvalidValue ) {
            std::cout << gazeSample.porFilteredX << " / " << gazeSample.porFilteredY;
        } else {
            std::cout << "invalid";
        }
        std::cout << std::endl;
    }
};

class EventReceiver : public elapi::ELApi::ELEventCallback
{
public:
    EventReceiver( elapi::ELApi& api )
    {
    }

    void STDCALL
    onEvent( elapi::ELApi::Event event ) override
    {
        switch ( event ) {
            case elapi::ELApi::Event::SCREEN_CHANGED:
                std::cout << "stimulus screen has changed" << std::endl;
                return;
            case elapi::ELApi::Event::CONNECTION_CLOSED:
                std::cout << "server has closed the connection" << std::endl;
                return;
            case elapi::ELApi::Event::DEVICE_CONNECTED:
                std::cout << "a new device has connected" << std::endl;
                return;
            case elapi::ELApi::Event::DEVICE_DISCONNECTED:
                std::cout << "device has disconnected" << std::endl;
                return;
            case elapi::ELApi::Event::TRACKING_STOPPED:
                std::cout << "tracking has stopped" << std::endl;
                return;
        }
    }
};

int
main( )
{
    // connect to server
    elapi::ELApi api( "Demo Client" );

    auto eventReceiver = std::make_unique< EventReceiver >( api );
    api.registerEventListener( eventReceiver.get( ) );

    const auto retConnect = api.connect( );
    if ( retConnect == elapi::ELApi::ReturnConnect::SUCCESS ) {
        std::cout << "connected to server" << std::endl;
    } else {
        std::cout << "Cannot connect to server: connect failed. Is the server running?"
                  << std::endl;
        close( );
        return 1;
    }

    // get screen infos
    elapi::ELApi::ScreenConfig screenConfig;
    api.getActiveScreen( screenConfig );
    std::cout << "screen resolution: " << screenConfig.resolutionX << "x"
              << screenConfig.resolutionY << std::endl;

    // get device infos
    elapi::ELApi::DeviceConfig deviceConfig;
    api.getDeviceConfig( deviceConfig );
    if ( deviceConfig.deviceSerial == 0 ) {
        std::cout << "no device connected" << std::endl;
        close( );
        return 1;
    }
    std::cout << "device found, available framerates: ";
    for ( int32_t i = 0; i < deviceConfig.numFrameRates; i++ ) {
        if ( i > 0 ) {
            std::cout << ", ";
        }
        std::cout << static_cast< int >( deviceConfig.frameRates[ i ] );
    }
    std::cout << std::endl;

    // start tracking
    const auto retStart = api.requestTracking( 0 );
    if ( retStart == elapi::ELApi::ReturnStart::SUCCESS ) {
        std::cout << "tracking started" << std::endl;
    } else {
        std::cout << "cannot start tracking: requestTracking failed" << std::endl;
        close( );
        return 1;
    }

    // calibrate
    const auto retCalibrate = api.calibrate( 0 );
    if ( retCalibrate == elapi::ELApi::ReturnCalibrate::SUCCESS ) {
        std::cout << "calibrated" << std::endl;
    } else {
        std::cout << "could not calibrate: calibrate failed" << std::endl;
        close( );
        return 1;
    }

    auto gazeSampleReceiver = std::make_unique< GazeSampleReceiver >( );
    api.registerGazeSampleListener( gazeSampleReceiver.get( ) );

    // wait for 10 seconds
    std::this_thread::sleep_for( std::chrono::seconds( 10 ) );

    api.disconnect( );
    api.registerGazeSampleListener( nullptr );
    api.registerEventListener( nullptr );
    eventReceiver      = nullptr;
    gazeSampleReceiver = nullptr;

    close( );
    return 0;
}
