﻿using System;
using System.Threading;
using eyelogic;

namespace democlientCs
{
    /**
     * Client demonstrating the use of remote connection.
     * Setup:
     *   - Computer A is running the EyeLogic Server. This is the workplace of the
     *     operator of the experiment.
     *   - Computer B is running the experiment. It is shown on a screen connected
     *     to computer B. This is the workplace of the participant.
     *   - The eye tracker is mounted on that screen (connected to computer B).
     *   - The eye tracker is wired to the USB port of computer A.
     *
     * In this setup, the client (running on computer B) has to connect to the
     * EyeLogic Server on computer A via the call connectRemote(). Both computers
     * need to be in the same local network.
     */

    public class DemoClientDualPc
    {
        /** EyeLogic API object */
        private ELCsApi m_api;

        static void Main( string[ ] args )
        {
            DemoClientDualPc client = new DemoClientDualPc( );
            client.run( );
        }

        void
        run( )
        {
            try {
                // initialize API
                System.Console.WriteLine( "initialize EyeLogic API..." );
                m_api = new ELCsApi( "C# Client" );
                System.Console.WriteLine( "    ... done" );

                // obtain list of available EyeLogic servers in the local network
                // Note: a server has to "enable remote" connections in order to be found
                ELCsApi.ServerInfo[ ] serverList = m_api.requestServerList( 120, 4 );
                if ( serverList == null ) {
                    System.Console.WriteLine(
                        "No server on local PC or in LAN found. Server must be running and " +
                        "has 'allow remote' connections enabled." );
                    m_api.destroy( );
                    System.Console.WriteLine( "press enter to quit" );
                    System.Console.ReadLine( );
                    return;
                }

                System.Console.WriteLine( "found {0} server(s):", serverList.Length );
                for ( int i = 0; i < serverList.Length; i++ ) {
                    System.Console.WriteLine( "[{0}] IP: {1}", i, serverList[ i ].ip );
                }

                // connect to EyeLogic Server
                System.Console.WriteLine( "connecting to server [0]" );
                m_api.connectRemote( serverList[ 0 ] );
                System.Console.WriteLine( "    ... done, connected" );

                // get available screens on this PC
                ELCsApi.ScreenConfig[ ] screenConfig = m_api.getAvailableScreens( );
                if ( screenConfig.Length == 0 ) {
                    System.Console.WriteLine( "no screens connected to local PC" );
                    m_api.disconnect( );
                    m_api.destroy( );
                    System.Console.WriteLine( "press enter to quit" );
                    System.Console.ReadLine( );
                    return;
                }
                System.Console.WriteLine( "found {0} screen(s):", screenConfig.Length );
                for ( int i = 0; i < screenConfig.Length; i++ ) {
                    System.Console.WriteLine( "[{0}] ID: {1} ({2}x{3})",
                                              i,
                                              screenConfig[ i ].id,
                                              screenConfig[ i ].resolutionX,
                                              screenConfig[ i ].resolutionY );
                }

                // set active screen
                ELCsApi.DeviceGeometry deviceGeometry =
                    new ELCsApi.DeviceGeometry { mmBelowScreen            = 10,
                                                 mmTrackerInFrontOfScreen = 10 };

                System.Console.WriteLine(
                    "Using screen [0]. Assuming eye tracker position (reference point): " +
                        "{0}mm below, {1}mm in front.",
                    deviceGeometry.mmBelowScreen,
                    deviceGeometry.mmTrackerInFrontOfScreen );
                m_api.setActiveScreen( screenConfig[ 0 ].id, deviceGeometry );

                // check connected eye tracker
                ELCsApi.DeviceConfig deviceConfig = m_api.getDeviceConfig( );
                if ( deviceConfig == null ) {
                    System.Console.WriteLine( "no device connected - exiting" );
                    m_api.disconnect( );
                    m_api.destroy( );
                    System.Console.WriteLine( "press enter to quit" );
                    System.Console.ReadLine( );
                    return;
                }
                System.Console.WriteLine( "device found, serial: {0}",
                                          deviceConfig.formatDeviceSerial( ) );

                // start tracking
                System.Console.WriteLine( "start tracking with default framerate" );
                m_api.requestTracking( 0 );
                System.Console.WriteLine( "    ... done" );

                // perform calibration
                System.Console.WriteLine( "start default calibration" );
                m_api.calibrate( 0 );
                System.Console.WriteLine( "    ... success" );

                // calibration successful
                // run your experiment here
                System.Console.WriteLine( "EyeLogic device is up and running" );

                // disconnect
                System.Console.WriteLine( "disconnecting..." );
                m_api.unrequestTracking( );
                m_api.disconnect( );
                m_api.destroy( );
                System.Console.WriteLine( "    ... done" );
            } catch ( ELException e ) {
                System.Console.WriteLine( e.Message + " (error ID: " + e.Error + ")" );
                if ( m_api != null ) {
                    m_api.disconnect( );
                    m_api.destroy( );
                }
            }

            System.Console.WriteLine( "press enter to quit" );
            System.Console.ReadLine( );
        }
    }
}
