# -----------------------------------------------------------------------
# Copyright (C) 2019-2023, EyeLogic GmbH
#
# Permission is hereby granted, free of charge, to any person or
# organization obtaining a copy of the software and accompanying
# documentation covered by this license (the "Software") to use,
# reproduce, display, distribute, execute, and transmit the Software,
# and to prepare derivative works of the Software, and to permit
# third-parties to whom the Software is furnished to do so.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE
# DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER
# LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------

#
# Client demonstrating the use of remote connection.
# Setup:
#   - Computer A is running the EyeLogic Server. This is the workplace of the
#     operator of the experiment.
#   - Computer B is running the experiment. It is shown on a screen connected
#     to computer B. This is the workplace of the participant.
#   - The eye tracker is mounted on that screen (connected to computer B).
#   - The eye tracker is wired to the USB port of computer A.
#
# In this setup, the client (running on computer B) has to connect to the
# EyeLogic Server on computer A via the call connectRemote(). Both computers
# need to be in the same local network.
#

from eyelogic.ELApi import *
import time

if __name__ == "__main__":

    # initialize API
    global ELInvalidValue
    api = ELApi("Demo Client")

    # obtain list of available EyeLogic servers in the local network
    # Note: a server has to "enable remote" connections in order to be found
    serverList = api.requestServerList(120, 4)
    if (len(serverList) == 0):
        print("No server on local PC or in LAN found. Server must be running and " +
              "has 'allow remote' connections enabled.",
              flush=True)
        input('Press Enter to exit')
        exit()

    print("found {} server(s):".format(len(serverList)))
    i = 0
    for server in serverList:
        print("{} IP: {}".format(i, server.ip))
        i += 1

    # connect to EyeLogic Server
    print("Connecting to server [0]")
    resultConnect = api.connectRemote(serverList[0])
    if (resultConnect != ELApi.ReturnConnect.SUCCESS):
        print("Connection to server failed: {}".format(resultConnect),
              flush=True)
        input('Press Enter to exit')
        exit()
    print("Successfully connected to server: {}".format(api.isConnected()),
          flush=True)

    # get available screens on this PC
    screens = api.getAvailableScreens()
    if (len(screens) == 0):
        print("No screens connected to local PC.",
              flush=True)
        input('Press Enter to exit')
        exit()
    print("found {} screen(s):".format(len(screens)))
    i = 0
    for screen in screens:
        print("[{}] ID: {} ({}x{})".format(i, screen.id,
            screen.resolutionX, screen.resolutionY))
        i += 1

    # set active screen
    deviceGeometry = ELApi.DeviceGeometry(10, 10)
    print("Using screen [0]. Assuming eye tracker position (reference point): " +
              "{}mm below, {}mm in front".format(deviceGeometry.mmBelowScreen,
              deviceGeometry.mmTrackerInFrontOfScreen))
    resultSetActiveScreen = api.setActiveScreen(screens[0].id, deviceGeometry)
    if (resultSetActiveScreen != ELApi.ReturnSetActiveScreen.SUCCESS):
        print("Setting active screen failed: {}".format(resultSetActiveScreen), flush=True)
        input('Press Enter to exit')
        exit()

    # check connected eye tracker
    deviceConfig = api.getDeviceConfig()
    if (deviceConfig.deviceSerial == 0):
        print("No device connected, will exit")
        input('Press Enter to exit')
        exit()
    else:
        print("Device found: framerates: {}".format(deviceConfig.frameRates),
              flush=True)

    # start tracking
    resultTracking = api.requestTracking(0)
    if (resultTracking != ELApi.ReturnStart.SUCCESS):
        print("Start tracking failed: {}".format(resultTracking), flush=True)
        input('Press Enter to exit')
        exit()

    # perform calibration
    resultCalibrate = api.calibrate(0)
    if (resultCalibrate != ELApi.ReturnCalibrate.SUCCESS):
        print("Calibration failed: {}".format(resultCalibrate), flush=True)
        input('Press Enter to exit')
        exit()

    # calibration successful
    # run your experiment here
    print("EyeLogic device is up and running")

    # disconnect
    print("disconnecting...")
    api.disconnect()

    input('Press Enter to exit')
